import z from 'zod';
import { UserId } from '../user/types';
import { PayoutMethodOriginType } from '../payoutMethod/types';
import { Currency } from '../currencies/types';
import { payoutBodyParser, payoutParamsParser, payoutResultParser, payoutRetryBodyParser } from './parsers';
import { Subset } from '../utils/types';
import { PAYOUT_QUEUE_STATUSES } from './constants';

enum PayoutBatchKeyBrand {
  _ = '',
}

// an uniq key per payout batch operation; serves as an idempotency key for a payout
export type PayoutBatchKey = string & PayoutBatchKeyBrand;

export type PayoutBatchItemCompositeKeySerialized = `${UserId}:${PayoutMethodOriginType}:${Currency}`;
export interface PayoutBatchItemCompositeKey<T extends PayoutMethodOriginType = PayoutMethodOriginType> {
  userId: UserId;
  originType: T;
  currency: Currency;
}

export type PayoutBody = z.infer<typeof payoutBodyParser>;
export type PayoutRetryBody = z.infer<typeof payoutRetryBodyParser>;
export type PayoutParams = z.infer<typeof payoutParamsParser>;
export type PayoutResult = z.infer<typeof payoutResultParser>;

// not the "whole args", doesnt include the batch key which isn't the part of the "args" language for idempotency
export type AuthorPayoutArgs = Partial<Record<PayoutBatchItemCompositeKeySerialized, number>>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertAuthorPayoutArgs1 = Subset<PayoutBody['items'], AuthorPayoutArgs>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertAuthorPayoutArgs2 = Subset<AuthorPayoutArgs, PayoutBody['items']>;

// pseudo status for the queue item; signifies "all done"/"any done"/"some done"
export type PayoutQueueItemStatus = typeof PAYOUT_QUEUE_STATUSES[number];
